use palette::Srgb;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct RGBDifference {
    pub r: Differ,
    pub g: Differ,
    pub b: Differ,
}

impl ColorDifference for Srgb {
    type Difference = RGBDifference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let r = other.red - self.red;
        let g = other.green - self.green;
        let b = other.blue - self.blue;

        RGBDifference {
            r: Differ {
                delta: r,
                percent: r / self.red,
            },
            g: Differ {
                delta: g,
                percent: g / self.green,
            },
            b: Differ {
                delta: b,
                percent: b / self.blue,
            },
        }
    }

    fn difference_relative(&self, other: &Self) -> Self::Difference {
        let r = if self.red == 0.0 {
            0.0
        } else {
            (other.red - self.red) / (1.0 - self.red)
        };
        let g = if self.green == 0.0 {
            0.0
        } else {
            (other.green - self.green) / (1.0 - self.green)
        };
        let b = if self.blue == 0.0 {
            0.0
        } else {
            (other.blue - self.blue) / (1.0 - self.blue)
        };

        RGBDifference {
            r: Differ {
                delta: other.red - self.red,
                percent: r,
            },
            g: Differ {
                delta: other.green - self.green,
                percent: g,
            },
            b: Differ {
                delta: other.blue - self.blue,
                percent: b,
            },
        }
    }
}
